import os

from .messages import printf

def center(__str: str, rm=0, lm=0):
    columns = os.get_terminal_size().columns
    rspaces = " " * rm
    lspaces = " " * lm

    for s in __str.splitlines():
        center_column = (columns - len(s)) // 2
        spaces = " "*center_column
        printf(spaces + lspaces + s + rspaces)

def right(__str: str, rm=0, lm=0):
    columns = os.get_terminal_size().columns
    rspaces = " " * rm
    lspaces = " " * lm


    for s in __str.splitlines():
        right_column = (columns - len(s))
        spaces = " "*right_column
        printf(spaces + lspaces + s + rspaces)



from dataclasses import dataclass

# ANSI 24 bit - RGB colors
def chex(hex: str):
    r,g,b = tuple(int(hex.lstrip("#")[i:i+2], 16) for i in (0, 2, 4))
    return "\033[38;2;{};{};{}m".format(r,g,b)

# ANSI 8 bit - 256 colors
def cansi(color: int):
    return "\033[38;5;{}m".format(color)

# ANSI 3/4 bit
def clegacy(color: int):
    return "\033[{}m".format(color)

def ansistr(__str: str, color: int):
    return cansi(color) + __str + cansi(0)

def legacy_ansistr(__str: str, color: int):
    return clegacy(color) + __str + cansi(0)


def hexstr(__str: str, hex: str):
    return chex(hex) + __str + "\033[0m"

@dataclass
class DefaultColors:
    black = cansi(0)
    red = cansi(1)
    green = cansi(2)
    yellow = cansi(3)
    blue = cansi(4)
    magenta = cansi(5)
    cyan = cansi(6)
    silver = cansi(7)
    grey = cansi(8)
    highred = cansi(9)
    highgreen = cansi(10)
    highyellow = cansi(11)
    highblue = cansi(12)
    highmagenta = cansi(13)
    highcyan = cansi(14)
    white = cansi(15)
    reset = "\033[0m"



from sys import stdout, stderr
import re

from . import color

colors = color.DefaultColors

_alert = colors.highred + "[!]" + colors.reset
_warn = colors.highyellow + "[@]" + colors.reset
_info = colors.cyan + "[?]" + colors.reset 
_success = colors.green + "[+]" + colors.reset
_error = colors.red + "[-]" + colors.reset 

auto_reset = False

def alertf(*objs, end="\n", sep=" ", file=stderr, flush=False):
    file.write(_alert + " " + sep.join(map(str, objs)) + end)
    if flush:
        file.flush()

def warnf(*objs, end="\n", start="", sep=" ", file=stderr, flush=False):
    file.write(start +_warn + " " + sep.join(map(str, objs)) + end)
    if flush:
        file.flush()

def infof(*objs, end="\n", start="", sep=" ", file=stderr, flush=False):
    file.write(start + _info + " " + sep.join(map(str, objs)) + end)
    if flush:
        file.flush()

def successf(*objs, end="\n", start="", sep=" ", file=stderr, flush=False):
    file.write(start + _success + " " + sep.join(map(str, objs)) + end)
    if flush:
        file.flush()
    

def errorf(*objs, end="\n", start="", sep=" ", file=stderr, flush=False):
    file.write(start + _error + " " + sep.join(map(str, objs)) + end)
    if flush:
        file.flush()

def printf(*objs, end="\n", sep=" ", file=stdout, flush=False):
    __text = sep.join(map(str, objs)) + end

    def color_replace(match):
        hex = match.group(1)

        if hex == "reset":
            return "\033[0m"
        else:
            return color.chex(hex)

    __text = re.sub(r"\{#([^}]+)\}", color_replace, __text)
    
    if auto_reset:
        __text += "\033[0m"

    file.write(__text) 
    
    if flush:
        file.flush()
    
